@@ -2,7 +2,6 @@
 Unit tests for the structured metamodel component.
 """
 import unittest
-import inspect
 
 import numpy as np
 from numpy.testing import assert_almost_equal
@@ -16,15 +15,14 @@ from openmdao.utils.testing_utils import use_tempdirs, force_check_partials
 
 scipy_gte_019 = True
 try:
-    from scipy.interpolate._bsplines import make_interp_spline
+    from scipy.interpolate._bsplines import make_interp_spline  # noqa: F401
 except ImportError:
     scipy_gte_019 = False
 
 # check that pyoptsparse is installed
 # if it is, try to use SNOPT but fall back to SLSQP
 OPT, OPTIMIZER = set_pyoptsparse_opt('SNOPT')
-if OPTIMIZER:
-    from openmdao.drivers.pyoptsparse_driver import pyOptSparseDriver
+
 
 x = np.array([-0.97727788, -0.15135721, -0.10321885,  0.40015721,  0.4105985,
                0.95008842,  0.97873798,  1.76405235,  1.86755799,  2.2408932 ])
@@ -627,10 +625,9 @@ class TestMetaModelStructuredScipy(unittest.TestCase):
 
         prob.run_model()
 
-        totals = prob.check_totals(of='comp.f', wrt=['tab.k', 'comp.p1', 'comp.p2', 'comp.p3'],
-                                   method='cs', out_stream=None);
-
-        assert_near_equal(totals['comp.f', 'tab.k']['abs error'][0], 0.0, tolerance=1e-10)
+        chk = prob.check_totals(of='comp.f', wrt=['tab.k', 'comp.p1', 'comp.p2', 'comp.p3'],
+                                method='cs', out_stream=None)
+        assert_check_totals(chk, atol=1e-10, rtol=1e-10)
 
     def test_training_gradient_setup_called_twice(self):
         model = om.Group()
@@ -1131,11 +1128,6 @@ class TestMetaModelStructuredPython(unittest.TestCase):
         model = om.Group()
         ivc = om.IndepVarComp()
 
-        mapdata = SampleMap()
-
-        params = mapdata.param_data
-        outs = mapdata.output_data
-
         ivc.add_output('x', np.array([.33]))
 
         ivc.add_output('f_train', np.array([.3, .7, .5, .6, .3, .4, .2]))
@@ -1210,10 +1202,6 @@ class TestMetaModelStructuredPython(unittest.TestCase):
     def test_training_gradient_unsupported(self):
         # If using a fixed table method
         model = om.Group()
-        mapdata = SampleMap()
-
-        params = mapdata.param_data
-        outs = mapdata.output_data
 
         comp = om.MetaModelStructuredComp(training_data_gradients=True, extrapolate=True,
                                           method='1D-akima', vec_size=1)
@@ -1235,10 +1223,7 @@ class TestMetaModelStructuredPython(unittest.TestCase):
 
     def test_vectorized_1D_akima(self):
         model = om.Group()
-        mapdata = SampleMap()
 
-        params = mapdata.param_data
-        outs = mapdata.output_data
         nn = 2
 
         comp = om.MetaModelStructuredComp(extrapolate=True,
@@ -1463,8 +1448,7 @@ class TestMetaModelStructuredCompFeature(unittest.TestCase):
         force_check_partials(prob, compact_print=True)
 
     def test_error_messages_scalar_only(self):
-        prob = om.Problem()
-        model = prob.model
+        om.Problem()
 
         comp = om.MetaModelStructuredComp(training_data_gradients=True,
                                           method='slinear', vec_size=3)
